# Honeybee: A Plugin for Environmental Analysis (GPL)
# This file is part of Honeybee.
#
# Copyright (c) 2021, Ladybug Tools.
# You should have received a copy of the GNU General Public License
# along with Honeybee; If not, see <http://www.gnu.org/licenses/>.
# 
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>

"""
Create settings for the EnergyPlus Shadow Calculation.
-

    Args:
        _solar_dist_: An integer or text desribing how EnergyPlus should treat beam solar
            radiation and reflectances from surfaces that strike the building surfaces.
            Default is "FullExteriorWithReflections". Choose from the following.
            * 0 = "MinimalShadowing" - In this case, exterior shadowing is only computed
                for windows and not for other opaque surfaces that might have their
                surface temperature affected by the sun. All beam solar radiation
                entering the zone is assumed to fall on the floor. A simple window
                view factor calculation will be used to distribute incoming diffuse
                solar energy between interior surfaces.
            * 1 = "FullExterior" - The simulation will perform the solar calculation
                in a manner that only accounts for direct sun and whether it is
                blocked by surrounding context geometry.  For the inside of the
                building, all beam solar radiation entering the zone is assumed
                to fall on the floor. A simple window view factor calculation will
                be used to distribute incoming diffuse solar energy between
                interior surfaces.
            * 2 = "FullInteriorAndExterior" - The simulation will perform the solar
                calculation in a manner that models the direct sun (and wheter it
                is blocked by outdoor context goemetry.  It will also ray trace
                the direct sun on the interior of zones to distribute it correctly
                between interior surfaces.  Any indirect sun or sun bouncing off
                of objects will not be modled.
            * 3 = "FullExteriorWithReflections" - The simulation will perform the
                solar calculation in a manner that accounts for both direct sun
                and the light bouncing off outdoor surrounding context.  For the
                inside of the building, all beam solar radiation entering the zone
                is assumed to fall on the floor. A simple window view factor
                calculation will be used to distribute incoming diffuse solar
                energy between interior surfaces.
            * 4 = "FullInteriorAndExteriorWithReflections" - The simulation will
                perform the solar calculation in a manner that accounts for light
                bounces that happen both outside and inside the zones.  This is the
                most accurate method and is the one assigned by default.  Note that,
                if you use this method, EnergyPlus will give Severe warnings if
                your zones have concave geometry (aka. are "L"-shaped).  Such
                geometries are not supported by this solar distribution calculation
                and it is recommeded that you either break up your zones in this
                case or not use this solar distribution method.
        _calc_method_: Text noting whether CPU-based polygon clipping method or GPU-based
            pixel counting method should be used. For low numbers of shading
            surfaces (less than ~200), PolygonClipping requires less runtime than
            PixelCounting. However, PixelCounting runtime scales significantly
            better at higher numbers of shading surfaces. PixelCounting also has
            no limitations related to zone concavity when used with any
            “FullInterior” solar distribution options. (Default: PolygonClipping).
            Choose from the following:
                * PolygonClipping
                * PixelCounting
        _update_method_: Text describing how often the solar and shading calculations are
            updated with respect to the flow of time in the simulation. (Default: Periodic)
            Choose from the following:
                * Periodic
                * Timestep
        _frequency_: Integer for the number of days in each period in
            which a unique shadow calculation will be performed. This field is only
            used if the AverageOverDaysInFrequency method is used in the previous
            field. Default - 30.
        _max_figures_: Integer for the number of figures used in shadow overlaps.
            Default - 15000.
    
    Returns:
        shadow_calc: A ShadowCalculation object that can be connected to the
            "HB Simulation Parameter" component in order to specify settings
            for the EnergyPlus Shadow Calculation.
"""

ghenv.Component.Name = 'HB Shadow Calculation'
ghenv.Component.NickName = 'ShadowCalc'
ghenv.Component.Message = '1.2.1'
ghenv.Component.Category = 'HB-Energy'
ghenv.Component.SubCategory = '5 :: Simulate'
ghenv.Component.AdditionalHelpFromDocStrings = '3'

try:
    from honeybee_energy.simulation.shadowcalculation import ShadowCalculation
except ImportError as e:
    raise ImportError('\nFailed to import honeybee_energy:\n\t{}'.format(e))


# dictionary to convert to acceptable solar distributions
SOLAR_DISTRIBUTIONS = {
    '0': 'MinimalShadowing',
    '1': 'FullExterior',
    '2': 'FullInteriorAndExterior',
    '3': 'FullExteriorWithReflections',
    '4': 'FullInteriorAndExteriorWithReflections',
    'MinimalShadowing': 'MinimalShadowing',
    'FullExterior': 'FullExterior',
    'FullInteriorAndExterior': 'FullInteriorAndExterior',
    'FullExteriorWithReflections': 'FullExteriorWithReflections',
    'FullInteriorAndExteriorWithReflections': 'FullInteriorAndExteriorWithReflections'
    }


# process the solar distribution
try:
    _solar_dist_ = SOLAR_DISTRIBUTIONS[_solar_dist_] if _solar_dist_ is not None \
        else 'FullExteriorWithReflections'
except KeyError:
    raise ValueError(' Input _solar_dist_ "{}" is not valid.\nChoose from the '
        'following:\n{}'.format(_solar_dist_, SOLAR_DISTRIBUTIONS.keys()))

# set other default values
_calc_method_ = _calc_method_ if _calc_method_ is not None else 'PolygonClipping'
_update_method_ = _update_method_ if _update_method_ is not None else 'Periodic'
_frequency_ = _frequency_ if _frequency_ is not None else 30
_max_figures_ = _max_figures_ if _max_figures_ is not None else 15000

# create the object
shadow_calc = ShadowCalculation(
    _solar_dist_, _calc_method_, _update_method_, _frequency_, _max_figures_)



# Honeybee: A Plugin for Environmental Analysis (GPL)
# This file is part of Honeybee.
#
# Copyright (c) 2021, Ladybug Tools.
# You should have received a copy of the GNU General Public License
# along with Honeybee; If not, see <http://www.gnu.org/licenses/>.
# 
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>

"""
Create a People object that can be used to create a ProgramType or be assigned
directly to a Room.
-

    Args:
        _name_: Text to set the name for the People and to be incorporated
            into a unique People identifier. If None, a unique name will
            be generated.
         _ppl_per_area: A numerical value for the number of people per square
            meter of floor area.
        _occupancy_sch: A fractional schedule for the occupancy over the course
            of the year. The fractional values in this schedule will get multiplied
            by the _people_per_area to yield a complete occupancy profile.
        _activity_sch_: A schedule for the activity of the occupants over the course
            of the year. The type limt of this schedule should be "Activity Level"
            and the values of the schedule equal to the number of Watts given off
            by an individual person in the room. If None, it will a default constant
            schedule with 120 Watts per person will be used, which is typical of
            awake, adult humans who are seated.

    Returns:
        people: A People object that can be used to create a ProgramType or
            be assigned directly to a Room.
"""

ghenv.Component.Name = "HB People"
ghenv.Component.NickName = 'People'
ghenv.Component.Message = '1.2.0'
ghenv.Component.Category = 'HB-Energy'
ghenv.Component.SubCategory = '3 :: Loads'
ghenv.Component.AdditionalHelpFromDocStrings = "3"

try:  # import the core honeybee dependencies
    from honeybee.typing import clean_and_id_ep_string, clean_ep_string
except ImportError as e:
    raise ImportError('\nFailed to import honeybee:\n\t{}'.format(e))

try:
    from honeybee_energy.load.people import People
    from honeybee_energy.lib.schedules import schedule_by_identifier
except ImportError as e:
    raise ImportError('\nFailed to import honeybee_energy:\n\t{}'.format(e))

try:
    from ladybug_rhino.grasshopper import all_required_inputs
except ImportError as e:
    raise ImportError('\nFailed to import ladybug_rhino:\n\t{}'.format(e))


if all_required_inputs(ghenv.Component):
    # make a default People name if none is provided
    name = clean_and_id_ep_string('People') if _name_ is None else \
        clean_ep_string(_name_)

    # get the schedules
    if isinstance(_occupancy_sch, str):
        _occupancy_sch = schedule_by_identifier(_occupancy_sch)
    if isinstance(_activity_sch_, str):
        _activity_sch_ = schedule_by_identifier(_activity_sch_)

    # create the People object
    people = People(name, _ppl_per_area, _occupancy_sch, _activity_sch_)
    if _name_ is not None:
        people.display_name = _name_



# Honeybee: A Plugin for Environmental Analysis (GPL)
# This file is part of Honeybee.
#
# Copyright (c) 2021, Ladybug Tools.
# You should have received a copy of the GNU General Public License
# along with Honeybee; If not, see <http://www.gnu.org/licenses/>.
# 
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>

"""
Create a ProgramType object by blending other ProgramTypes together using a weighted
average based on program ratios.
-

    Args:
        _name_: Text to set the name for the ProgramType and to be incorporated
            into a unique ProgramType identifier.
        _programs: A list of ProgramType objects that will be averaged
            together to make a new ProgramType.
        _ratios_: A list of fractional numbers with the same length as the input
            programs that sum to 1. These will be used to weight each of the
            ProgramType objects in the resulting average. If None, the input
            program objects will be weighted equally. Default: None.

    Returns:
        program: A ProgramType object that's a weighted average between the
            input _programs.
"""

ghenv.Component.Name = 'HB Blend ProgramTypes'
ghenv.Component.NickName = 'BlendPrograms'
ghenv.Component.Message = '1.2.1'
ghenv.Component.Category = 'HB-Energy'
ghenv.Component.SubCategory = '0 :: Basic Properties'
ghenv.Component.AdditionalHelpFromDocStrings = '2'

try:  # import the core honeybee dependencies
    from honeybee.typing import clean_and_id_ep_string, clean_ep_string
except ImportError as e:
    raise ImportError('\nFailed to import honeybee:\n\t{}'.format(e))

try:
    from honeybee_energy.programtype import ProgramType
    from honeybee_energy.lib.programtypes import program_type_by_identifier, \
        building_program_type_by_identifier
except ImportError as e:
    raise ImportError('\nFailed to import honeybee_energy:\n\t{}'.format(e))

try:
    from ladybug_rhino.grasshopper import all_required_inputs
except ImportError as e:
    raise ImportError('\nFailed to import ladybug_rhino:\n\t{}'.format(e))


if all_required_inputs(ghenv.Component):
    # set default ratios to None
    _ratios_ = _ratios_ if len(_ratios_) != 0 else None
    name = clean_and_id_ep_string('ProgramType') if _name_ is None else \
        clean_ep_string(_name_)

    # get programs from library if a name is input
    for i, prog in enumerate(_programs):
        if isinstance(prog, str):
            try:
                _programs[i] = building_program_type_by_identifier(prog)
            except ValueError:
                _programs[i] = program_type_by_identifier(prog)

    # create blended program
    program = ProgramType.average(name, _programs, _ratios_)
    if _name_ is not None:
        program.display_name = _name_


# Honeybee: A Plugin for Environmental Analysis (GPL)
# This file is part of Honeybee.
#
# Copyright (c) 2021, Ladybug Tools.
# You should have received a copy of the GNU General Public License
# along with Honeybee; If not, see <http://www.gnu.org/licenses/>.
# 
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>

"""
Create a Ventilation object that can be used to create a ProgramType or be
assigned directly to a Room.
_
Note the the 4 ventilation types (_flow_per_person_, _flow_per_area_, _flow_per_zone_,
_ach_) are ultimately summed together to yeild the ventilation design flow rate used
in the simulation.
-

    Args:
        _name_: Text to set the name for the Ventilation and to be incorporated
            into a unique Ventilation identifier. If None, a unique name will
            be generated.
        _flow_per_person_: A numerical value for the intensity of ventilation
            in m3/s per person. Note that setting this value here does not mean
            that ventilation is varied based on real-time occupancy but rather
            that the design level of ventilation is determined using this value
            and the People object of the zone. To vary ventilation in real time,
            the ventilation schedule should be used. Most ventilation standards
            support that a value of 0.01 m3/s (10 L/s or ~20 cfm) per person is
            sufficient to remove odors. Accordingly, setting this value to 0.01
            and using 0 for the following ventilation terms will often be suitable
            for many applications. Default: 0.
        _flow_per_area_: A numerical value for the intensity of ventilation in m3/s
            per square meter of floor area. Default: 0.
        _flow_per_zone_: A numerical value for the design level of ventilation
            in m3/s for the entire zone. Default: 0.
        _ach_: A numberical value for the design level of ventilation
            in air changes per hour (ACH) for the entire zone. This is particularly
            helpful for hospitals, where ventilation standards are often given
            in ACH. Default: 0.
        _schedule_: An optional fractional schedule for the ventilation over the
            course of the year. The fractional values will get multiplied by
            the total design flow rate (determined from the fields above and the
            calculation_method) to yield a complete ventilation profile. Setting
            this schedule to be the occupancy schedule of the zone will mimic demand
            controlled ventilation. If None, the design level of ventilation will
            be used throughout all timesteps of the simulation. Default: None.

    Returns:
        vent: An Ventilation object that can be used to create a ProgramType or
            be assigned directly to a Room.
"""

ghenv.Component.Name = 'HB Ventilation'
ghenv.Component.NickName = 'Ventilation'
ghenv.Component.Message = '1.2.0'
ghenv.Component.Category = 'HB-Energy'
ghenv.Component.SubCategory = '3 :: Loads'
ghenv.Component.AdditionalHelpFromDocStrings = '3'

try:  # import the core honeybee dependencies
    from honeybee.typing import clean_and_id_ep_string, clean_ep_string
except ImportError as e:
    raise ImportError('\nFailed to import honeybee:\n\t{}'.format(e))

try:
    from honeybee_energy.load.ventilation import Ventilation
    from honeybee_energy.lib.schedules import schedule_by_identifier
except ImportError as e:
    raise ImportError('\nFailed to import honeybee_energy:\n\t{}'.format(e))

try:
    from ladybug_rhino.grasshopper import all_required_inputs
except ImportError as e:
    raise ImportError('\nFailed to import ladybug_rhino:\n\t{}'.format(e))


# make a default Ventilation name if none is provided
name = clean_and_id_ep_string('Ventilation') if _name_ is None else \
    clean_ep_string(_name_)

# get the schedule
if isinstance(_schedule_, str):
    _schedule_ = schedule_by_identifier(_schedule_)

# get default _flow_per_person_, _flow_per_area_, and _ach_
_flow_per_person_ = _flow_per_person_ if _flow_per_person_ is not None else 0.0
_flow_per_area_ = _flow_per_area_ if _flow_per_area_ is not None else 0.0
_flow_per_zone_ = _flow_per_zone_ if _flow_per_zone_ is not None else 0.0
_ach_ = _ach_ if _ach_ is not None else 0.0

# create the Ventilation object
vent = Ventilation(name, _flow_per_person_, _flow_per_area_,
                    _flow_per_zone_, _ach_, _schedule_)
if _name_ is not None:
    vent.display_name = _name_



# Honeybee: A Plugin for Environmental Analysis (GPL)
# This file is part of Honeybee.
#
# Copyright (c) 2021, Ladybug Tools.
# You should have received a copy of the GNU General Public License
# along with Honeybee; If not, see <http://www.gnu.org/licenses/>.
# 
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>

"""
Apply schedules to a Room or ProgramType.
_
Note that, if a schedule is assigned to a Room or ProgramType that posseses
no value for a given load, an error will be raised. For example, assigning a
gas_equip_sch_ to a Room that has no GasEquipment object associated with it.
This situation can be avoided by first passing the Rooms or ProgramTypes
through the "HB Apply Load Values" component to eastablish a value for a
given load.
-

    Args:
        _room_or_program: Honeybee Rooms or Honeybee ProgramType objects for which
            schedules should be changed. This can also be the identifier of a
            ProgramType to be looked up in the program type library.
        occupancy_sch_: A fractional schedule for the occupancy over the course
            of the year. This can also be the identifier of a schedule to be looked
            up in the schedule library.
        activity_sch_: A schedule for the activity of the occupants over the course of the
            year. The type limt of this schedule should be "ActivityLevel"
            and the values of the schedule equal to the number of Watts given off
            by an individual person in the room. If None, it will a default constant
            schedule with 120 Watts per person will be used, which is typical of
            awake, adult humans who are seated.
         heating_setpt_sch_: A temperature schedule for the heating setpoint.
            This can also be a identifier of a schedule to be looked up in the
            schedule library. The type limit of this schedule should be
            temperature and the values should be the temperature setpoint in
            degrees Celcius.
        cooling_setpt_sch_: A temperature schedule for the cooling setpoint.
            This can also be a identifier of a schedule to be looked up in the
            schedule library. The type limit of this schedule should be
            temperature and the values should be the temperature setpoint in
            degrees Celcius.
        lighting_sch_: A fractional for the use of lights over the course of the year.
            This can also be the identifier of a schedule to be looked up in the
            schedule library.
        electric_equip_sch_: A fractional for the use of electric equipment over
            the course of the year. This can also be the identifier of a schedule to
            be looked up in the schedule library.
        gas_equip_sch_: A fractional for the use of gas equipment over
            the course of the year. This can also be the identifier of a schedule to
            be looked up in the schedule library.
        infiltration_sch_: A fractional schedule for the infiltration over the
            course of the year. This can also be the identifier of a schedule to
            be looked up in the schedule library.
        ventilation_sch_: An optional fractional schedule for the ventilation over the
            course of the year. This can also be the identifier of a schedule to be
            looked up in the schedule library. The fractional values will get
            multiplied by the total design flow rate (determined from the fields
            above and the calculation_method) to yield a complete ventilation
            profile. Setting this schedule to be the occupancy schedule of the
            zone will mimic demand controlled ventilation.

    Returns:
        report: Reports, errors, warnings, etc.
        mod_obj: The input Rooms or ProgramTypes with their schedules modified.
"""

ghenv.Component.Name = "HB Apply Room Schedules"
ghenv.Component.NickName = 'ApplyRoomSch'
ghenv.Component.Message = '1.2.2'
ghenv.Component.Category = 'HB-Energy'
ghenv.Component.SubCategory = '2 :: Schedules'
ghenv.Component.AdditionalHelpFromDocStrings = "3"

import uuid

try:
    from honeybee.room import Room
except ImportError as e:
    raise ImportError('\nFailed to import honeybee:\n\t{}'.format(e))

try:
    from honeybee_energy.lib.schedules import schedule_by_identifier
    from honeybee_energy.lib.programtypes import program_type_by_identifier, \
        building_program_type_by_identifier
    from honeybee_energy.programtype import ProgramType
except ImportError as e:
    raise ImportError('\nFailed to import honeybee_energy:\n\t{}'.format(e))

try:
    from ladybug_rhino.grasshopper import all_required_inputs, longest_list
except ImportError as e:
    raise ImportError('\nFailed to import ladybug_rhino:\n\t{}'.format(e))


def schedule_object(schedule):
    """Get a schedule object by its identifier or return it it it's already a schedule."""
    if isinstance(schedule, str):
        return schedule_by_identifier(schedule)
    return schedule


def dup_load(hb_obj, object_name, input_name):
    """Duplicate a load object assigned to a Room or ProgramType."""
    # try to get the load object assgined to the Room or ProgramType
    try:  # assume it's a Room
        load_obj = hb_obj.properties
        for attribute in ('energy', object_name):
            load_obj = getattr(load_obj, attribute)
    except AttributeError:  # it's a ProgramType
        load_obj = getattr(hb_obj, object_name)

    try:  # duplicate the load object
        dup_load = load_obj.duplicate()
        dup_load.identifier = '{}_{}'.format(hb_obj.identifier, object_name)
        return dup_load
    except AttributeError:
        raise ValueError(
            '{0} has been input but the Room or ProgramType posseses no {1} object.'
            '\nUse the "HB Apply Load Values" component to define a {1} '
            'object.'.format(input_name, object_name))


def assign_load(hb_obj, load_obj, object_name):
    """Assign a load object to a Room or a ProgramType."""
    try:  # assume it's a Room
        setattr(hb_obj.properties.energy, object_name, load_obj)
    except AttributeError:  # it's a ProgramType
        setattr(hb_obj, object_name, load_obj)


def duplicate_and_id_program(program):
    """Duplicate a program and give it a new unique ID."""
    new_prog = program.duplicate()
    new_prog.identifier = '{}_{}'.format(program.identifier, str(uuid.uuid4())[:8])
    return new_prog


if all_required_inputs(ghenv.Component):
    # duplicate the initial objects
    mod_obj = []
    for obj in _room_or_program:
        if isinstance(obj, Room):
            mod_obj.append(obj.duplicate())
        elif isinstance(obj, ProgramType):
            mod_obj.append(duplicate_and_id_program(obj))
        elif isinstance(obj, str):
            try:
                program = building_program_type_by_identifier(obj)
            except ValueError:
                program = program_type_by_identifier(obj)
            mod_obj.append(duplicate_and_id_program(program))
        else:
            raise TypeError('Expected Honeybee Room or ProgramType. '
                            'Got {}.'.format(type(obj)))

    # assign the occupancy schedule
    if len(occupancy_sch_) != 0:
        for i, obj in enumerate(mod_obj):
            people = dup_load(obj, 'people', 'occupancy_sch_')
            people.occupancy_schedule = schedule_object(longest_list(occupancy_sch_, i))
            assign_load(obj, people, 'people')

    # assign the activity schedule
    if len(activity_sch_) != 0:
        for i, obj in enumerate(mod_obj):
            people = dup_load(obj, 'people', 'activity_sch_')
            people.activity_schedule = schedule_object(longest_list(activity_sch_, i))
            assign_load(obj, people, 'people')

    # assign the lighting schedule
    if len(lighting_sch_) != 0:
        for i, obj in enumerate(mod_obj):
            lighting = dup_load(obj, 'lighting', 'lighting_sch_')
            lighting.schedule = schedule_object(longest_list(lighting_sch_, i))
            assign_load(obj, lighting, 'lighting')

    # assign the electric equipment schedule
    if len(electric_equip_sch_) != 0:
        for i, obj in enumerate(mod_obj):
            equip = dup_load(obj, 'electric_equipment', 'electric_equip_sch_')
            equip.schedule = schedule_object(longest_list(electric_equip_sch_, i))
            assign_load(obj, equip, 'electric_equipment')

    # assign the gas equipment schedule
    if len(gas_equip_sch_) != 0:
        for i, obj in enumerate(mod_obj):
            equip = dup_load(obj, 'gas_equipment', 'gas_equip_sch_')
            equip.schedule = schedule_object(longest_list(gas_equip_sch_, i))
            assign_load(obj, equip, 'gas_equipment')

    # assign the infiltration schedule
    if len(infiltration_sch_) != 0:
        for i, obj in enumerate(mod_obj):
            infiltration = dup_load(obj, 'infiltration', 'infiltration_sch_')
            infiltration.schedule = schedule_object(longest_list(infiltration_sch_, i))
            assign_load(obj, infiltration, 'infiltration')

    # assign the ventilation schedule
    if len(ventilation_sch_) != 0:
        for i, obj in enumerate(mod_obj):
            ventilation = dup_load(obj, 'ventilation', 'ventilation_sch_')
            ventilation.schedule = schedule_object(longest_list(ventilation_sch_, i))
            assign_load(obj, ventilation, 'ventilation')

    # assign the heating setpoint schedule
    if len(heating_setpt_sch_) != 0:
        for i, obj in enumerate(mod_obj):
            setpoint = dup_load(obj, 'setpoint', 'heating_setpt_sch_')
            setpoint.heating_schedule = schedule_object(longest_list(heating_setpt_sch_, i))
            assign_load(obj, setpoint, 'setpoint')

    # assign the cooling setpoint schedule
    if len(cooling_setpt_sch_) != 0:
        for i, obj in enumerate(mod_obj):
            setpoint = dup_load(obj, 'setpoint', 'cooling_setpt_sch_')
            setpoint.cooling_schedule = schedule_object(longest_list(cooling_setpt_sch_, i))
            assign_load(obj, setpoint, 'setpoint')



# Honeybee: A Plugin for Environmental Analysis (GPL)
# This file is part of Honeybee.
#
# Copyright (c) 2021, Ladybug Tools.
# You should have received a copy of the GNU General Public License
# along with Honeybee; If not, see <http://www.gnu.org/licenses/>.
# 
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>

"""
Create parameters with criteria for sizing the heating and cooling system.
-

    Args:
        ddy_file_: An optional path to a .ddy file on your system, which contains
            information about the design days used to size the hvac system. If None,
            honeybee will look for a .ddy file next to the .epw and extract all
            99.6% and 0.4% design days.
        filter_ddays_: Boolean to note whether the design days in the ddy_file_
            should be filtered to only include 99.6% and 0.4% design days. If None
            or False, all design days in the ddy_file_ will be incorporated into
            the sizing parameters. This can also be the integer 2 to filter for
            99.0% and 1.0% design days.
        _heating_fac_: A number that will get multiplied by the peak heating load
            for each zone in the model in order to size the heating system for
            the model. Must be greater than 0. Default: 1.25.
        _cooling_fac_: A number that will get multiplied by the peak cooling load
            for each zone in the model in order to size the cooling system for
            the model. Must be greater than 0. Default: 1.15.
    
    Returns:
        sizing: Parameters with criteria for sizing the heating and cooling system.
            These can be connected to the "HB Simulation Parameter" component in
            order to specify settings for the EnergyPlus simulation.
"""

ghenv.Component.Name = 'HB Sizing Parameter'
ghenv.Component.NickName = 'SizingPar'
ghenv.Component.Message = '1.2.0'
ghenv.Component.Category = 'HB-Energy'
ghenv.Component.SubCategory = '5 :: Simulate'
ghenv.Component.AdditionalHelpFromDocStrings = '3'

try:
    from honeybee_energy.simulation.sizing import SizingParameter
except ImportError as e:
    raise ImportError('\nFailed to import honeybee_energy:\n\t{}'.format(e))


# set default sizing factors
heating_fac = 1.25 if _heating_fac_ is None else _heating_fac_
cooling_fac = 1.15 if _cooling_fac_ is None else _cooling_fac_

# create the object
sizing = SizingParameter(None, heating_fac, cooling_fac)

# apply design days from ddy
if ddy_file_ is not None:
    if filter_ddays_ == 1:
        sizing.add_from_ddy_996_004(ddy_file_)
    elif filter_ddays_ == 2:
        sizing.add_from_ddy_990_010(ddy_file_)
    else:
        sizing.add_from_ddy(ddy_file_)



# Honeybee: A Plugin for Environmental Analysis (GPL)
# This file is part of Honeybee.
#
# Copyright (c) 2021, Ladybug Tools.
# You should have received a copy of the GNU General Public License
# along with Honeybee; If not, see <http://www.gnu.org/licenses/>.
# 
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>

"""
Create a simulation parameter object that carries a complete set of EnergyPlus
simulation settings and can be plugged into the "HB Model To OSM" component.
-

    Args:
        _north_: A number between -360 and 360 for the counterclockwise
            difference between the North and the positive Y-axis in degrees.
            90 is West and 270 is East. Note that this is different than the
            convention used in EnergyPlus, which uses clockwise difference
            instead of counterclockwise difference. This can also be Vector
            for the direction to North. (Default: 0)
        _output_: A SimulationOutput that lists the desired outputs from the
            simulation and the format in which to report them. This can be
            created using the "HB Simulation Output" component. Default is to
            request zone energy use at an hourly timestep.
        _run_period_: A ladybyg AnalysisPeriod object to describe the time period
            over which to run the simulation. the default is to run the simulation
            for the whole year.
        daylight_saving_: An optional ladybug AnalysisPeriod object to describe
            start and end of daylight savings time in the simulation. If None, no
            daylight savings time will be applied to the simulation.
        holidays_: A list of Ladybug Date objects for the holidays within the
            simulation. These should be in the format of 'DD Month' (eg. '1 Jan',
            '25 Dec'). If None, no holidays are applied. Default: None.
        _start_dow_: Text for the day of the week on which the simulation
            starts. Default: 'Sunday'. Choose from the following:
                * Sunday
                * Monday
                * Tuesday
                * Wednesday
                * Thursday
                * Friday
                * Saturday
        _timestep_: An integer for the number of timesteps per hour at which the
            calculation will be run. Default: 6. The following values are acceptable:
            (1, 2, 3, 4, 5, 6, 10, 12, 15, 20, 30, 60)
        _terrain_: Text for the terrain type in which the model sits, used to determine
            the wind profile. Default: 'City'. Choose from: 
                * Ocean
                * Country
                * Suburbs
                * Urban
                * City
        _sim_control_: A SimulationControl object that describes which types
            of calculations to run. This can be generated from the "HB
            Simulation Control" component. Default: perform a sizing calculation
            but only run the simulation for the RunPeriod.
        _shadow_calc_: A ShadowCalculation object describing settings for the
            EnergyPlus Shadow Calculation. This can be generated from the "HB
            Shadow Calculation" component. Default: Average over 30 days with
            FullExteriorWithReflections.
        _sizing_: A SizingParameter object with criteria for sizing the heating
            and cooling system.  This can be generated from the "HB Sizing
            Parameter" component.

    Returns:
        sim_par: A SimulationParameter object that can be connected to the
            "HB Model To IDF" component in order to specify EnergyPlus
            simulation settings
"""

ghenv.Component.Name = 'HB Simulation Parameter'
ghenv.Component.NickName = 'SimPar'
ghenv.Component.Message = '1.2.0'
ghenv.Component.Category = 'HB-Energy'
ghenv.Component.SubCategory = '5 :: Simulate'
ghenv.Component.AdditionalHelpFromDocStrings = '3'

try:
    from ladybug.dt import Date, DateTime
except ImportError as e:
    raise ImportError('\nFailed to import ladybug:\n\t{}'.format(e))

try:
    from honeybee_energy.simulation.output import SimulationOutput
    from honeybee_energy.simulation.runperiod import RunPeriod
    from honeybee_energy.simulation.daylightsaving import DaylightSavingTime
    from honeybee_energy.simulation.parameter import SimulationParameter
except ImportError as e:
    raise ImportError('\nFailed to import honeybee_energy:\n\t{}'.format(e))

try:  # import the ladybug_rhino dependencies
    from ladybug_rhino.togeometry import to_vector2d
except ImportError as e:
    raise ImportError('\nFailed to import ladybug_rhino:\n\t{}'.format(e))


# set default simulation outputs
if _output_ is None:
    _output_ = SimulationOutput()
    _output_.add_zone_energy_use()
    _output_.add_hvac_energy_use()

# set default simulation run period
_run_period_ = RunPeriod.from_analysis_period(_run_period_) \
    if _run_period_ is not None else RunPeriod()

# set the daylight savings if it is input
if daylight_saving_ is not None:
    daylight_saving = DaylightSavingTime.from_analysis_period(daylight_saving_)
    _run_period_.daylight_saving_time = daylight_saving

# set the holidays if requested.
if len(holidays_) != 0:
    try:
        dates = tuple(Date.from_date_string(date) for date in holidays_)
    except ValueError:
        dates = tuple(DateTime.from_date_time_string(date).date for date in holidays_)
    _run_period_.holidays = dates

# set the start day of the week if it is input
if _start_dow_ is not None:
    _run_period_.start_day_of_week = _start_dow_.title()

# set the default timestep
_timestep_ = _timestep_ if _timestep_ is not None else 6

# set the default timestep
_terrain_ = _terrain_.title() if _terrain_ is not None else 'City'

# return final simulation parameters
sim_par = SimulationParameter(output=_output_,
                              run_period=_run_period_,
                              timestep=_timestep_,
                              simulation_control=_sim_control_,
                              shadow_calculation=_shadow_calc_,
                              sizing_parameter=_sizing_,
                              terrain_type=_terrain_)

# set the north if it is not defaulted
if _north_ is not None:
    try:
        sim_par.north_vector = to_vector2d(_north_)
    except AttributeError:  # north angle instead of vector
        sim_par.north_angle = float(_north_)
